/*
 * Dynamixel.c
 *
 *  Created on: 20 ene. 2021
 *      Author: Josue
 */


/*
 Dynamixel.cpp - Ax-12+ Half Duplex USART Comunication
 Copyright (c) 2011 Savage Electronics.
 Created by Josue Alejandro Gutierrez on 27/01/11.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 *****************************************************************************
 Modifications:

 25/07/2011 - Eliminado la modificacion serial para ser modificada dentro del mismo Hardware Serial.
 25/07/2011 - Modificado la funcion setBD() para aceptar todas la velocidades sin PDF.
 25/07/2011 - Agregada la funcion de Rotacion Continua.
 26/07/2011 - Agregada la funcion begin sin seteo de Direction_Pin.
 25/07/2011 - Agregada la funcion Reset.
 26/07/2011 - Agregada la funcion Reg_Write en move y moveSpeed.
 26/07/2011 - Agregada la funcion Action.
 13/12/2011 - Arreglado el manejo y envio de variables.
 22/12/2011 - Compatible con la actualizacion Arduino 1.0.
 10/01/2012 - Utilizacion de Macros y eliminacion codigo no necesario.
 11/01/2012 - Agregadas las funciones:
              int setTempLimit(unsigned char ID, unsigned char Temperature);
              int setAngleLimit(unsigned char ID, int CWLimit, int CCWLimit);
              int setVoltageLimit(unsigned char ID, unsigned char DVoltage, unsigned char UVoltage);
			  int setMaxTorque(unsigned char ID, int MaxTorque);
              int setSRL(unsigned char ID, unsigned char SRL);
              int setRDT(unsigned char ID, unsigned char RDT);
              int setLEDAlarm(unsigned char ID, unsigned char LEDAlarm);
              int setShutdownAlarm(unsigned char ID, unsigned char SALARM);
              int setCMargin(unsigned char ID, unsigned char CWCMargin, unsigned char CCWCMargin);
			  int setCSlope(unsigned char ID, unsigned char CWCSlope, unsigned char CCWCSlope);
 15/01/2012 - Agregadas las funciones:
              int setPunch(unsigned char ID, int Punch);
              int moving(unsigned char ID);
              int lockRegister(unsigned char ID);
			  int RWStatus(unsigned char ID);
              int readSpeed(unsigned char ID);
              int readLoad(unsigned char ID);
 08/07/2019
			- Se unifican todas las variantes de la biblioteca en una sola
			- Se agrega funcion de reset general

 TODO:


 *****************************************************************************

 Contact: Josue Alejandro Gutierrez - Josue.Gutierrez@savageelectronics.com
 Web:     https://www.savageelectrtonics.com/
 Autor:   Josue Alejandro Gutierrez

 */

#include <Dynamixel.h>
#include "Delay.h"
#include "UART.h"

// Macro for the selection of the Serial Port

#define uartInit(args)	(UART0Init(args))	  // UART Initialization
#define sendData(args)  (UART0Write(args))    // Write Over Serial
#define availableData() (UART0Available())    // Check Serial Data Available
#define readData()      (UART0Read())         // Read Serial Data
#define peekData()      (UART0Peek())         // Peek Serial Data
#define flushBuffer()   (UART0Flush())        // Flush Serial Data

	unsigned char headerStart;
	unsigned char responseID;
	unsigned char packetLenght;
	unsigned char Checksum;
	unsigned char Direction_Pin;
	unsigned char Time_Counter;
	unsigned char Incoming_Byte;
	unsigned char Register_High_Byte;
	unsigned char Register_Low_Byte;
	int RegisterByte;
	int Error_Byte;

unsigned short generateCRC(unsigned char *dataPacket, unsigned short packetSize)
	{
	    unsigned short i, j,crc = 0;

	    unsigned short crc_table[256] = {
	        0x0000, 0x8005, 0x800F, 0x000A, 0x801B, 0x001E, 0x0014, 0x8011,
	        0x8033, 0x0036, 0x003C, 0x8039, 0x0028, 0x802D, 0x8027, 0x0022,
	        0x8063, 0x0066, 0x006C, 0x8069, 0x0078, 0x807D, 0x8077, 0x0072,
	        0x0050, 0x8055, 0x805F, 0x005A, 0x804B, 0x004E, 0x0044, 0x8041,
	        0x80C3, 0x00C6, 0x00CC, 0x80C9, 0x00D8, 0x80DD, 0x80D7, 0x00D2,
	        0x00F0, 0x80F5, 0x80FF, 0x00FA, 0x80EB, 0x00EE, 0x00E4, 0x80E1,
	        0x00A0, 0x80A5, 0x80AF, 0x00AA, 0x80BB, 0x00BE, 0x00B4, 0x80B1,
	        0x8093, 0x0096, 0x009C, 0x8099, 0x0088, 0x808D, 0x8087, 0x0082,
	        0x8183, 0x0186, 0x018C, 0x8189, 0x0198, 0x819D, 0x8197, 0x0192,
	        0x01B0, 0x81B5, 0x81BF, 0x01BA, 0x81AB, 0x01AE, 0x01A4, 0x81A1,
	        0x01E0, 0x81E5, 0x81EF, 0x01EA, 0x81FB, 0x01FE, 0x01F4, 0x81F1,
	        0x81D3, 0x01D6, 0x01DC, 0x81D9, 0x01C8, 0x81CD, 0x81C7, 0x01C2,
	        0x0140, 0x8145, 0x814F, 0x014A, 0x815B, 0x015E, 0x0154, 0x8151,
	        0x8173, 0x0176, 0x017C, 0x8179, 0x0168, 0x816D, 0x8167, 0x0162,
	        0x8123, 0x0126, 0x012C, 0x8129, 0x0138, 0x813D, 0x8137, 0x0132,
	        0x0110, 0x8115, 0x811F, 0x011A, 0x810B, 0x010E, 0x0104, 0x8101,
	        0x8303, 0x0306, 0x030C, 0x8309, 0x0318, 0x831D, 0x8317, 0x0312,
	        0x0330, 0x8335, 0x833F, 0x033A, 0x832B, 0x032E, 0x0324, 0x8321,
	        0x0360, 0x8365, 0x836F, 0x036A, 0x837B, 0x037E, 0x0374, 0x8371,
	        0x8353, 0x0356, 0x035C, 0x8359, 0x0348, 0x834D, 0x8347, 0x0342,
	        0x03C0, 0x83C5, 0x83CF, 0x03CA, 0x83DB, 0x03DE, 0x03D4, 0x83D1,
	        0x83F3, 0x03F6, 0x03FC, 0x83F9, 0x03E8, 0x83ED, 0x83E7, 0x03E2,
	        0x83A3, 0x03A6, 0x03AC, 0x83A9, 0x03B8, 0x83BD, 0x83B7, 0x03B2,
	        0x0390, 0x8395, 0x839F, 0x039A, 0x838B, 0x038E, 0x0384, 0x8381,
	        0x0280, 0x8285, 0x828F, 0x028A, 0x829B, 0x029E, 0x0294, 0x8291,
	        0x82B3, 0x02B6, 0x02BC, 0x82B9, 0x02A8, 0x82AD, 0x82A7, 0x02A2,
	        0x82E3, 0x02E6, 0x02EC, 0x82E9, 0x02F8, 0x82FD, 0x82F7, 0x02F2,
	        0x02D0, 0x82D5, 0x82DF, 0x02DA, 0x82CB, 0x02CE, 0x02C4, 0x82C1,
	        0x8243, 0x0246, 0x024C, 0x8249, 0x0258, 0x825D, 0x8257, 0x0252,
	        0x0270, 0x8275, 0x827F, 0x027A, 0x826B, 0x026E, 0x0264, 0x8261,
	        0x0220, 0x8225, 0x822F, 0x022A, 0x823B, 0x023E, 0x0234, 0x8231,
	        0x8213, 0x0216, 0x021C, 0x8219, 0x0208, 0x820D, 0x8207, 0x0202
	    };

	    for(j = 0; j < packetSize; j++)
	    {
	        i = ((unsigned short)(crc >> 8) ^ dataPacket[j]) & 0xFF;
	        crc = (crc << 8) ^ crc_table[i];
	    }

	    return crc;
	}


void switchCom(unsigned char DirPin,unsigned char Mode){
	if (Mode){
		GPIOCTRL->PSOR |= (1<<CTRLPIN);
	}else{
		GPIOCTRL->PCOR |= (1<<CTRLPIN);
	}
}

int readRegister(unsigned char numBytes){//unsigned int ID){

	RegisterByte = -128;

	Time_Counter = 0;
	while((availableData() < 7 + numBytes) & (Time_Counter < TIME_OUT)){
		Time_Counter++;
		delayus(DLYTIME);
	}

	while (availableData() > 2 + numBytes){
		Incoming_Byte = readData();
		if ( (Incoming_Byte == 255) & (peekData() == 255) ){
			headerStart = readData();                       // Start Bytes
			responseID = readData();                        // Ax-12 ID
			packetLenght = readData();                      // Length
			if( (Error_Byte = readData()) != 0 ){           // Error
				return (Error_Byte*(-1));}

			if (numBytes == 1){
				RegisterByte = readData();        			// Register
			}else if (numBytes == 2){
				Register_Low_Byte = readData();            // Position Bytes
				Register_High_Byte = readData();
				RegisterByte =  Register_High_Byte << 8;
				RegisterByte += Register_Low_Byte;
			}
			Checksum = readData();
		}
	}
	return (RegisterByte);									   //Return Register Value
}

int readError(){//unsigned int ID){

	Time_Counter = 0;
	while((availableData() < 7) & (Time_Counter < TIME_OUT)){  // Wait for Data
		Time_Counter++;
		delayus(DLYTIME);
	}

	while (availableData() > 2){
		Incoming_Byte = readData();
		if ( (Incoming_Byte == 255) & (peekData() == 255) ){
			headerStart = readData();                         // Start Bytes
			responseID = readData();                          // Ax-12 ID
			packetLenght = readData();                        // Length
			Error_Byte = readData();                          // Error
			Checksum = readData();							  // Checksum
			return (Error_Byte);
		}
	}
	return (-128);											 // No Ax Response ( Use bit 7 )
}

void Dynamixelinit(unsigned int baudrate){

	SIM->SCGC5 |= SIM_SCGC5_PORTA(1);			// PORTE Clock enable
	PORTCTRL->PCR[CTRLPIN] |= PORT_PCR_MUX(1);
	GPIOCTRL->PDDR |= (1<<CTRLPIN);				// Control pin for data direction

	initUART0(baudrate);		// UART Initialization
}

int Dynamixelreset(unsigned char ID)
{
	Checksum = (~(ID + AX_RESET_LENGTH + AX_RESET))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_RESET_LENGTH);
	sendData(AX_RESET);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int Dynamixelping(unsigned char ID, unsigned char protocol)
{
	Checksum = (~(ID + AX_READ_DATA + AX_PING))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	if (protocol == 2){
		sendData(0xFD);
		sendData(0x00);
		sendData(ID);
		sendData(0x03);
		sendData(0x00);
		sendData(0x01);
		sendData(0x19);
		sendData(0x4e);
		switchCom(Direction_Pin,Rx_MODE);
	}else if (protocol == 1){
		sendData(ID);
		sendData(AX_READ_DATA);
		sendData(AX_PING);
		sendData(Checksum);
		switchCom(Direction_Pin,Rx_MODE);
	}

    return (readError());
}

int DynamixelsetID(unsigned char ID, unsigned char newID)
{
	Checksum = (~(ID + AX_ID_LENGTH + AX_WRITE_DATA + AX_ID + newID))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_ID_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_ID);
    sendData(newID);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetBD(unsigned char ID, long baud)
{
	unsigned char Baud_Rate = (2000000/baud) - 1;
    Checksum = (~(ID + AX_BD_LENGTH + AX_WRITE_DATA + AX_BAUD_RATE + Baud_Rate))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_BD_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_BAUD_RATE);
    sendData(Baud_Rate);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int Dynamixelmove(unsigned char ID, int Position,unsigned char Dynamixelprotocol)
{
    unsigned short CRC_Bytes;
    unsigned char CRC_lowByte, CRC_highByte;
    char Position_H,Position_L;
    Position_H = Position >> 8;           // 16 bits - 2 x 8 bits variables
    Position_L = Position;
	Checksum = (~(ID + AX_GOAL_LENGTH + AX_WRITE_DATA + AX_GOAL_POSITION_L + Position_L + Position_H))&0xFF;

    unsigned char dataPacket[14] = {DNMXL_H1,DNMXL_H2,DNMXL_H3,DNMXL_RSVD,ID,DNMXL_GOAL_LENGTH_LOW,DNMXL_GOAL_LENGTH_HIGH,DNMXL_WRITE_DATA,DNMXL_GOAL_LOW,DNMXL_GOAL_HIGH,Position_L,Position_H,0x00,0x00};

    CRC_Bytes = generateCRC(dataPacket,12);
    CRC_lowByte = (CRC_Bytes & 0x00FF);
    CRC_highByte = (CRC_Bytes >> 8) & 0x00FF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(DNMXL_H1);
    sendData(DNMXL_H2);
    if (Dynamixelprotocol == 2){
    	sendData(DNMXL_H3);
    	sendData(DNMXL_RSVD);
    	sendData(ID);
    	sendData(DNMXL_GOAL_LENGTH_LOW);
    	sendData(DNMXL_GOAL_LENGTH_HIGH);
    	sendData(DNMXL_WRITE_DATA);
    	sendData(DNMXL_GOAL_LOW);
    	sendData(DNMXL_GOAL_HIGH);
    	sendData(Position_L);
    	sendData(Position_H);
    	sendData(CRC_lowByte);
    	sendData(CRC_highByte);
    }else if(Dynamixelprotocol == 1){
    	sendData(ID);
    	sendData(AX_GOAL_LENGTH);
    	sendData(AX_WRITE_DATA);
    	sendData(AX_GOAL_POSITION_L);
    	sendData(Position_L);
    	sendData(Position_H);
    	sendData(Checksum);
    }
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelmoveSpeed(unsigned char ID, int Position, int Speed,unsigned char Dynamixelprotocol)
{
	unsigned short CRC_Bytes;
	unsigned char CRC_lowByte, CRC_highByte;
    char Position_H,Position_L,Speed_H,Speed_L;
    Position_H = Position >> 8;
    Position_L = Position;                // 16 bits - 2 x 8 bits variables
    Speed_H = Speed >> 8;
    Speed_L = Speed;                      // 16 bits - 2 x 8 bits variables
	Checksum = (~(ID + AX_GOAL_SP_LENGTH + AX_WRITE_DATA + AX_GOAL_POSITION_L + Position_L + Position_H + Speed_L + Speed_H))&0xFF;
	unsigned char dataPacket[16] = {DNMXL_H1,DNMXL_H2,DNMXL_H3,DNMXL_RSVD,ID,DNMXL_GOALSPEED_LENGTH_LOW,DNMXL_GOALSPEED_LENGTH_HIGH,DNMXL_WRITE_DATA,DNMXL_GOAL_LOW,DNMXL_GOAL_HIGH,Position_L,Position_H,Speed_L,Speed_H,0x00,0x00};

	CRC_Bytes = generateCRC(dataPacket,14);
	CRC_lowByte = (CRC_Bytes & 0x00FF);
	CRC_highByte = (CRC_Bytes >> 8) & 0x00FF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(DNMXL_H1);
	sendData(DNMXL_H2);
	if (Dynamixelprotocol == 2){
		sendData(DNMXL_H3);
		sendData(DNMXL_RSVD);
		sendData(ID);
		sendData(DNMXL_GOALSPEED_LENGTH_LOW);
		sendData(DNMXL_GOALSPEED_LENGTH_HIGH);
		sendData(DNMXL_WRITE_DATA);
		sendData(DNMXL_GOAL_LOW);
		sendData(DNMXL_GOAL_HIGH);
		sendData(Position_L);
		sendData(Position_H);
		sendData(Speed_L);
		sendData(Speed_H);
		sendData(CRC_lowByte);
		sendData(CRC_highByte);
	}else if(Dynamixelprotocol == 1){
		sendData(ID);
		sendData(AX_GOAL_SP_LENGTH);
		sendData(AX_WRITE_DATA);
		sendData(AX_GOAL_POSITION_L);
		sendData(Position_L);
		sendData(Position_H);
		sendData(Speed_L);
		sendData(Speed_H);
		sendData(Checksum);
	}
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetEndless(unsigned char ID, bool Status)
{
 if ( Status ) {
	  char AX_CCW_AL_LT = 0;     // Changing the CCW Angle Limits for Full Rotation.
	  Checksum = (~(ID + AX_GOAL_LENGTH + AX_WRITE_DATA + AX_CCW_ANGLE_LIMIT_L))&0xFF;

	  switchCom(Direction_Pin,Tx_MODE);
      sendData(AX_START);
      sendData(AX_START);
      sendData(ID);
      sendData(AX_GOAL_LENGTH);
      sendData(AX_WRITE_DATA);
      sendData(AX_CCW_ANGLE_LIMIT_L );
      sendData(AX_CCW_AL_LT);
      sendData(AX_CCW_AL_LT);
      sendData(Checksum);
	  switchCom(Direction_Pin,Rx_MODE);

	  return(readError());
 }
 else
 {
	 Dynamixelturn(ID,0,0);
	 Checksum = (~(ID + AX_GOAL_LENGTH + AX_WRITE_DATA + AX_CCW_ANGLE_LIMIT_L + AX_CCW_AL_L + AX_CCW_AL_H))&0xFF;

	 switchCom(Direction_Pin,Tx_MODE);
	 sendData(AX_START);
	 sendData(AX_START);
	 sendData(ID);
	 sendData(AX_GOAL_LENGTH);
	 sendData(AX_WRITE_DATA);
	 sendData(AX_CCW_ANGLE_LIMIT_L);
	 sendData(AX_CCW_AL_L);
	 sendData(AX_CCW_AL_H);
	 sendData(Checksum);
	 switchCom(Direction_Pin,Rx_MODE);

	 return (readError());
  }
 }

int Dynamixelturn(unsigned char ID, bool SIDE, int Speed)
{
		if (SIDE == 0){                          // Move Left///////////////////////////

			char Speed_H,Speed_L;
			Speed_H = Speed >> 8;
			Speed_L = Speed;                     // 16 bits - 2 x 8 bits variables
			Checksum = (~(ID + AX_SPEED_LENGTH + AX_WRITE_DATA + AX_GOAL_SPEED_L + Speed_L + Speed_H))&0xFF;

			switchCom(Direction_Pin,Tx_MODE);
			sendData(AX_START);
			sendData(AX_START);
			sendData(ID);
			sendData(AX_SPEED_LENGTH);
			sendData(AX_WRITE_DATA);
			sendData(AX_GOAL_SPEED_L);
			sendData(Speed_L);
			sendData(Speed_H);
			sendData(Checksum);
			switchCom(Direction_Pin,Rx_MODE);

			return(readError());
		}
		else
		{                                            // Move Rigth////////////////////
			char Speed_H,Speed_L;
			Speed_H = (Speed >> 8) + 4;
			Speed_L = Speed;                     // 16 bits - 2 x 8 bits variables
			Checksum = (~(ID + AX_SPEED_LENGTH + AX_WRITE_DATA + AX_GOAL_SPEED_L + Speed_L + Speed_H))&0xFF;

			switchCom(Direction_Pin,Tx_MODE);
			sendData(AX_START);
			sendData(AX_START);
			sendData(ID);
			sendData(AX_SPEED_LENGTH);
			sendData(AX_WRITE_DATA);
			sendData(AX_GOAL_SPEED_L);
			sendData(Speed_L);
			sendData(Speed_H);
			sendData(Checksum);
			switchCom(Direction_Pin,Rx_MODE);

			return(readError());
		}
}

int DynamixelmoveRW(unsigned char ID, int Position)
{
    char Position_H,Position_L;
    Position_H = Position >> 8;           // 16 bits - 2 x 8 bits variables
    Position_L = Position;
    Checksum = (~(ID + AX_GOAL_LENGTH + AX_REG_WRITE + AX_GOAL_POSITION_L + Position_L + Position_H))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_GOAL_LENGTH);
    sendData(AX_REG_WRITE);
    sendData(AX_GOAL_POSITION_L);
    sendData(Position_L);
    sendData(Position_H);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelmoveSpeedRW(unsigned char ID, int Position, int Speed)
{
    char Position_H,Position_L,Speed_H,Speed_L;
    Position_H = Position >> 8;
    Position_L = Position;                // 16 bits - 2 x 8 bits variables
    Speed_H = Speed >> 8;
    Speed_L = Speed;                      // 16 bits - 2 x 8 bits variables
    Checksum = (~(ID + AX_GOAL_SP_LENGTH + AX_REG_WRITE + AX_GOAL_POSITION_L + Position_L + Position_H + Speed_L + Speed_H))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_GOAL_SP_LENGTH);
    sendData(AX_REG_WRITE);
    sendData(AX_GOAL_POSITION_L);
    sendData(Position_L);
    sendData(Position_H);
    sendData(Speed_L);
    sendData(Speed_H);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

void Dynamixelaction()
{
	Checksum = (~(BROADCAST_ID + AX_ACTION_LENGTH + AX_ACTION))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(BROADCAST_ID);
    sendData(AX_ACTION_LENGTH);
    sendData(AX_ACTION);
    sendData(AX_ACTION_CHECKSUM);
	switchCom(Direction_Pin,Rx_MODE);
}

int DynamixeltorqueStatus( unsigned char ID, bool Status,unsigned char Dynamixelprotocol)
{
    unsigned short CRC_Bytes;
    unsigned char CRC_lowByte, CRC_highByte;

    Checksum = (~(ID + AX_TORQUE_LENGTH + AX_WRITE_DATA + AX_TORQUE_ENABLE + Status))&0xFF;
    unsigned char dataPacket[13] = {DNMXL_H1,DNMXL_H2,DNMXL_H3,DNMXL_RSVD,ID,DNMXL_TRQ_LENGTH_LOW,DNMXL_TRQ_LENGTH_HIGH,DNMXL_WRITE_DATA,DNMXL_TRQ_LOW,DNMXL_TRQ_HIGH,Status,0x00,0x00};

    CRC_Bytes = generateCRC(dataPacket,11);
    CRC_lowByte = (CRC_Bytes & 0x00FF);
    CRC_highByte = (CRC_Bytes >> 8) & 0x00FF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(DNMXL_H1);
	sendData(DNMXL_H2);
    if (Dynamixelprotocol == 2){
    	sendData(DNMXL_H3);
    	sendData(DNMXL_RSVD);
    	sendData(ID);
    	sendData(DNMXL_TRQ_LENGTH_LOW);
    	sendData(DNMXL_TRQ_LENGTH_HIGH);
    	sendData(DNMXL_WRITE_DATA);
    	sendData(DNMXL_TRQ_LOW);
    	sendData(DNMXL_TRQ_HIGH);
    	sendData(Status);
    	sendData(CRC_lowByte);
    	sendData(CRC_highByte);
    } else if(Dynamixelprotocol == 1){
    	sendData(ID);
    	sendData(AX_TORQUE_LENGTH);
    	sendData(AX_WRITE_DATA);
    	sendData(AX_TORQUE_ENABLE);
    	sendData(Status);
    	sendData(Checksum);
    }
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelledStatus(unsigned char ID, unsigned char LEDColor, unsigned char Dynamixelprotocol)
{
    unsigned short CRC_Bytes;
    unsigned char CRC_lowByte, CRC_highByte;

    Checksum = (~(ID + AX_LED_LENGTH + AX_WRITE_DATA + AX_LED + (LEDColor & 0x01))) & 0xFF;
    unsigned char dataPacket[13] = {DNMXL_H1,DNMXL_H2,DNMXL_H3,DNMXL_RSVD,ID,DNMXL_LED_LENGTH_LOW,DNMXL_LED_LENGTH_HIGH,DNMXL_WRITE_DATA,DNMXL_LED_LOW,DNMXL_LED_HIGH,LEDColor,0x00,0x00};

    CRC_Bytes = generateCRC(dataPacket,11);
    CRC_lowByte = (CRC_Bytes & 0x00FF);
    CRC_highByte = (CRC_Bytes >> 8) & 0x00FF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(DNMXL_H1);
    sendData(DNMXL_H2);
    if (Dynamixelprotocol == 2){
    	sendData(DNMXL_H3);
    	sendData(DNMXL_RSVD);
    	sendData(ID);
    	sendData(DNMXL_LED_LENGTH_LOW);
    	sendData(DNMXL_LED_LENGTH_HIGH);
    	sendData(DNMXL_WRITE_DATA);
    	sendData(DNMXL_LED_LOW);
    	sendData(DNMXL_LED_HIGH);
    	sendData(LEDColor);
    	sendData(CRC_lowByte);
    	sendData(CRC_highByte);
    }else if(Dynamixelprotocol == 1){
    	sendData(ID);
    	sendData(AX_LED_LENGTH);
    	sendData(AX_WRITE_DATA);
    	sendData(AX_LED);
    	sendData(LEDColor & 0x01);
    	sendData(Checksum);
    }
    switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetTempLimit(unsigned char ID, unsigned char Temperature)
{
	Checksum = (~(ID + AX_TL_LENGTH +AX_WRITE_DATA+ AX_LIMIT_TEMPERATURE + Temperature))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_TL_LENGTH);
	sendData(AX_WRITE_DATA);
	sendData(AX_LIMIT_TEMPERATURE);
    sendData(Temperature);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetVoltageLimit(unsigned char ID, unsigned char DVoltage, unsigned char UVoltage)
{
	Checksum = (~(ID + AX_VL_LENGTH +AX_WRITE_DATA+ AX_DOWN_LIMIT_VOLTAGE + DVoltage + UVoltage))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_VL_LENGTH);
	sendData(AX_WRITE_DATA);
	sendData(AX_DOWN_LIMIT_VOLTAGE);
    sendData(DVoltage);
    sendData(UVoltage);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetAngleLimit(unsigned char ID, int CWLimit, int CCWLimit)
{
	char CW_H,CW_L,CCW_H,CCW_L;
    CW_H = CWLimit >> 8;
    CW_L = CWLimit;                // 16 bits - 2 x 8 bits variables
    CCW_H = CCWLimit >> 8;
    CCW_L = CCWLimit;
	Checksum = (~(ID + AX_VL_LENGTH +AX_WRITE_DATA+ AX_CW_ANGLE_LIMIT_L + CW_H + CW_L + AX_CCW_ANGLE_LIMIT_L + CCW_H + CCW_L))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_CCW_CW_LENGTH);
	sendData(AX_WRITE_DATA);
	sendData(AX_CW_ANGLE_LIMIT_L);
    sendData(CW_L);
	sendData(CW_H);
	sendData(AX_CCW_ANGLE_LIMIT_L);
    sendData(CCW_L);
	sendData(CCW_H);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetMaxTorque(unsigned char ID, int MaxTorque)
{
    char MaxTorque_H,MaxTorque_L;
    MaxTorque_H = MaxTorque >> 8;           // 16 bits - 2 x 8 bits variables
    MaxTorque_L = MaxTorque;
	Checksum = (~(ID + AX_MT_LENGTH + AX_WRITE_DATA + AX_MAX_TORQUE_L + MaxTorque_L + MaxTorque_H))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_MT_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_MAX_TORQUE_L);
    sendData(MaxTorque_L);
    sendData(MaxTorque_H);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetSRL(unsigned char ID, unsigned char SRL)
{
	Checksum = (~(ID + AX_SRL_LENGTH + AX_WRITE_DATA + AX_RETURN_LEVEL + SRL))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_SRL_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_RETURN_LEVEL);
    sendData(SRL);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetRDT(unsigned char ID, unsigned char RDT)
{
	Checksum = (~(ID + AX_RDT_LENGTH + AX_WRITE_DATA + AX_RETURN_DELAY_TIME + (RDT/2)))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_RDT_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_RETURN_DELAY_TIME);
    sendData((RDT/2));
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetLEDAlarm(unsigned char ID, unsigned char LEDAlarm)
{
	Checksum = (~(ID + AX_LEDALARM_LENGTH + AX_WRITE_DATA + AX_ALARM_LED + LEDAlarm))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_LEDALARM_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_ALARM_LED);
    sendData(LEDAlarm);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetShutdownAlarm(unsigned char ID, unsigned char SALARM)
{
	Checksum = (~(ID + AX_SALARM_LENGTH + AX_ALARM_SHUTDOWN + AX_ALARM_LED + SALARM))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_SALARM_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_ALARM_SHUTDOWN);
    sendData(SALARM);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetCMargin(unsigned char ID, unsigned char CWCMargin, unsigned char CCWCMargin)
{
	Checksum = (~(ID + AX_CM_LENGTH +AX_WRITE_DATA+ AX_CW_COMPLIANCE_MARGIN + CWCMargin + AX_CCW_COMPLIANCE_MARGIN + CCWCMargin))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_CM_LENGTH);
	sendData(AX_WRITE_DATA);
	sendData(AX_CW_COMPLIANCE_MARGIN);
    sendData(CWCMargin);
	sendData(AX_CCW_COMPLIANCE_MARGIN);
    sendData(CCWCMargin);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetCSlope(unsigned char ID, unsigned char CWCSlope, unsigned char CCWCSlope)
{
	Checksum = (~(ID + AX_CS_LENGTH +AX_WRITE_DATA+ AX_CW_COMPLIANCE_SLOPE + CWCSlope + AX_CCW_COMPLIANCE_SLOPE + CCWCSlope))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
	sendData(AX_START);
	sendData(AX_START);
	sendData(ID);
	sendData(AX_CS_LENGTH);
	sendData(AX_WRITE_DATA);
	sendData(AX_CW_COMPLIANCE_SLOPE);
    sendData(CWCSlope);
	sendData(AX_CCW_COMPLIANCE_SLOPE);
    sendData(CCWCSlope);
	sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelsetPunch(unsigned char ID, int Punch)
{
    char Punch_H,Punch_L;
    Punch_H = Punch >> 8;           // 16 bits - 2 x 8 bits variables
    Punch_L = Punch;
	Checksum = (~(ID + AX_PUNCH_LENGTH + AX_WRITE_DATA + AX_PUNCH_L + Punch_L + Punch_H))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_PUNCH_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_PUNCH_L);
    sendData(Punch_L);
    sendData(Punch_H);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int Dynamixelmoving(unsigned char ID)
{
    Checksum = (~(ID + AX_MOVING_LENGTH  + AX_READ_DATA + AX_MOVING + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_MOVING_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_MOVING);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read moving status
}

int DynamixellockRegister(unsigned char ID)
{
	Checksum = (~(ID + AX_LR_LENGTH + AX_WRITE_DATA + AX_LOCK + LOCK))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
	sendData(AX_LR_LENGTH);
    sendData(AX_WRITE_DATA);
    sendData(AX_LOCK);
    sendData(LOCK);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

    return (readError());
}

int DynamixelRWStatus(unsigned char ID)
{
    Checksum = (~(ID + AX_RWS_LENGTH  + AX_READ_DATA + AX_REGISTERED_INSTRUCTION + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_RWS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_REGISTERED_INSTRUCTION);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the RWS
}

int DynamixelreadTemperature(unsigned char ID, unsigned char Dynamixelprotocol)
{
	unsigned short CRC_Bytes;
	unsigned char CRC_lowByte, CRC_highByte;

    Checksum = (~(ID + AX_TEM_LENGTH  + AX_READ_DATA + AX_PRESENT_TEMPERATURE + AX_BYTE_READ))&0xFF;
    unsigned char dataPacket[14] = {DNMXL_H1,DNMXL_H2,DNMXL_H3,DNMXL_RSVD,ID,DNMXL_TEMP_LENGTH_LOW,DNMXL_TEMP_LENGTH_HIGH,DNMXL_READ_DATA,DNMXL_TEMP_LOW,DNMXL_TEMP_HIGH,DNMXL_BYTE_LOW,DNMXL_BYTE_HIGH,0x00,0x00};

    CRC_Bytes = generateCRC(dataPacket,12);
    CRC_lowByte = (CRC_Bytes & 0x00FF);
    CRC_highByte = (CRC_Bytes >> 8) & 0x00FF;

    switchCom(Direction_Pin,Tx_MODE);
    sendData(DNMXL_H1);
    sendData(DNMXL_H2);
    if (Dynamixelprotocol == 2){
    	sendData(DNMXL_H3);
    	sendData(DNMXL_RSVD);
    	sendData(ID);
    	sendData(DNMXL_TEMP_LENGTH_LOW);
    	sendData(DNMXL_TEMP_LENGTH_HIGH);
    	sendData(DNMXL_READ_DATA);
    	sendData(DNMXL_TEMP_LOW);
    	sendData(DNMXL_TEMP_HIGH);
    	sendData(DNMXL_BYTE_LOW);
    	sendData(DNMXL_BYTE_HIGH);
    	sendData(CRC_lowByte);
    	sendData(CRC_highByte);
    }else if(Dynamixelprotocol == 1){
    	sendData(ID);
    	sendData(AX_TEM_LENGTH);
    	sendData(AX_READ_DATA);
    	sendData(AX_PRESENT_TEMPERATURE);
    	sendData(AX_BYTE_READ);
    	sendData(Checksum);
    }
    switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read temperature
}

int DynamixelreadTemperatureLimit(unsigned char ID)
{
    Checksum = (~(ID + AX_TEM_LENGTH  + AX_READ_DATA + AX_LIMIT_TEMPERATURE + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_TEM_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_LIMIT_TEMPERATURE);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read temperature
}

int DynamixelreadVoltage(unsigned char ID)
{
    Checksum = (~(ID + AX_VOLT_LENGTH  + AX_READ_DATA + AX_PRESENT_VOLTAGE + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_VOLT_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_PRESENT_VOLTAGE);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read Voltage
}

int DynamixelreadMinVoltage(unsigned char ID)
{
    Checksum = (~(ID + AX_VOLT_LENGTH  + AX_READ_DATA + AX_DOWN_LIMIT_VOLTAGE + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_VOLT_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_DOWN_LIMIT_VOLTAGE);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read Voltage
}

int DynamixelreadMaxVoltage(unsigned char ID)
{
    Checksum = (~(ID + AX_VOLT_LENGTH  + AX_READ_DATA + AX_UP_LIMIT_VOLTAGE + AX_BYTE_READ))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_VOLT_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_UP_LIMIT_VOLTAGE);
    sendData(AX_BYTE_READ);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(1));               // Returns the read Voltage
}

int DynamixelreadRegister(unsigned char ID,unsigned char Register, unsigned char bytes)
{
    Checksum = (~(ID + AX_REG_LENGTH  + AX_READ_DATA + Register + bytes))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_REG_LENGTH);
    sendData(AX_READ_DATA);
    sendData(Register);
    sendData(bytes);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(bytes));               // Returns the read register
}

int DynamixelreadSpeed(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_PRESENT_SPEED_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_PRESENT_SPEED_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read speed
}

int DynamixelreadLoad(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_PRESENT_LOAD_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_PRESENT_LOAD_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read load
}

int DynamixelreadMaxTorque(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_MAX_TORQUE_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_MAX_TORQUE_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read torque
}

int DynamixelreadPosition(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_PRESENT_POSITION_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_PRESENT_POSITION_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read position
}

int DynamixelreadCWAngle(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_CW_ANGLE_LIMIT_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_CW_ANGLE_LIMIT_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read CW Angle
}

int DynamixelreadCCWAngle(unsigned char ID)
{
    Checksum = (~(ID + AX_POS_LENGTH  + AX_READ_DATA + AX_CCW_ANGLE_LIMIT_L + AX_BYTE_READ_POS))&0xFF;

	switchCom(Direction_Pin,Tx_MODE);
    sendData(AX_START);
    sendData(AX_START);
    sendData(ID);
    sendData(AX_POS_LENGTH);
    sendData(AX_READ_DATA);
    sendData(AX_CCW_ANGLE_LIMIT_L);
    sendData(AX_BYTE_READ_POS);
    sendData(Checksum);
	switchCom(Direction_Pin,Rx_MODE);

	return (readRegister(2));     // Returns the read CCW Angle
}



