/*
 Dynamixel.cpp - Ax-12+ Half Duplex USART Communication
 Copyright (c) 2011 Savage Electronics.
 Created by Josue Alejandro Gutierrez on 27/01/11.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 Contact: Josue Alejandro Gutierrez - Josue.Gutierrez@savageelectronics.com
 Web:     https://www.savageelectrtonics.com/
 Autor:   Josue Alejandro Gutierrez

 */

#ifndef Dynamixel_h
#define Dynamixel_h

#include "MK26F18.h"
#include <stdbool.h>

#define PORTCTRL	PORTB
#define GPIOCTRL	GPIOB
#define CTRLPIN		10

#define DLYTIME		15000

	// EEPROM AREA  ///////////////////////////////////////////////////////////
#define AX_MODEL_NUMBER_L           0
#define AX_MODEL_NUMBER_H           1
#define AX_VERSION                  2
#define AX_ID                       3
#define AX_BAUD_RATE                4
#define AX_RETURN_DELAY_TIME        5
#define AX_CW_ANGLE_LIMIT_L         6
#define AX_CW_ANGLE_LIMIT_H         7
#define AX_CCW_ANGLE_LIMIT_L        8
#define AX_CCW_ANGLE_LIMIT_H        9
#define AX_SYSTEM_DATA2             10
#define AX_LIMIT_TEMPERATURE        11
#define AX_DOWN_LIMIT_VOLTAGE       12
#define AX_UP_LIMIT_VOLTAGE         13
#define AX_MAX_TORQUE_L             14
#define AX_MAX_TORQUE_H             15
#define AX_RETURN_LEVEL             16
#define AX_ALARM_LED                17
#define AX_ALARM_SHUTDOWN           18
#define AX_OPERATING_MODE           19
#define AX_DOWN_CALIBRATION_L       20
#define AX_DOWN_CALIBRATION_H       21
#define AX_UP_CALIBRATION_L         22
#define AX_UP_CALIBRATION_H         23

	// RAM AREA  //////////////////////////////////////////////////////////////
#define AX_TORQUE_ENABLE            24
#define AX_LED                      25
#define AX_CW_COMPLIANCE_MARGIN     26
#define AX_CCW_COMPLIANCE_MARGIN    27
#define AX_CW_COMPLIANCE_SLOPE      28
#define AX_CCW_COMPLIANCE_SLOPE     29
#define AX_GOAL_POSITION_L          30
#define AX_GOAL_POSITION_H          31
#define AX_GOAL_SPEED_L             32
#define AX_GOAL_SPEED_H             33
#define AX_TORQUE_LIMIT_L           34
#define AX_TORQUE_LIMIT_H           35
#define AX_PRESENT_POSITION_L       36
#define AX_PRESENT_POSITION_H       37
#define AX_PRESENT_SPEED_L          38
#define AX_PRESENT_SPEED_H          39
#define AX_PRESENT_LOAD_L           40
#define AX_PRESENT_LOAD_H           41
#define AX_PRESENT_VOLTAGE          42
#define AX_PRESENT_TEMPERATURE      43
#define AX_REGISTERED_INSTRUCTION   44
#define AX_PAUSE_TIME               45
#define AX_MOVING                   46
#define AX_LOCK                     47
#define AX_PUNCH_L                  48
#define AX_PUNCH_H                  49

    // Status Return Levels ///////////////////////////////////////////////////////////////
#define AX_RETURN_NONE              0
#define AX_RETURN_READ              1
#define AX_RETURN_ALL               2

    // Instruction Set ///////////////////////////////////////////////////////////////
#define AX_PING                     1
#define AX_READ_DATA                2
#define AX_WRITE_DATA               3
#define AX_REG_WRITE                4
#define AX_ACTION                   5
#define AX_RESET                    6
#define AX_SYNC_WRITE               131

#define DNMXL_READ_DATA				0x02
#define DNMXL_WRITE_DATA			0x03

	// Specials ///////////////////////////////////////////////////////////////

#define DNMXL_H1                    	0xFF
#define DNMXL_H2                    	0xFF
#define DNMXL_H3                    	0xFD
#define DNMXL_RSVD						0x00

#define DNMXL_TRQ_LENGTH_LOW			0x06
#define DNMXL_TRQ_LENGTH_HIGH			0x00
#define DNMXL_LED_LENGTH_LOW			0x06
#define DNMXL_LED_LENGTH_HIGH			0x00
#define DNMXL_GOAL_LENGTH_LOW			0x07
#define DNMXL_GOAL_LENGTH_HIGH			0x00
#define DNMXL_GOALSPEED_LENGTH_LOW		0x09
#define DNMXL_GOALSPEED_LENGTH_HIGH		0x00
#define DNMXL_TEMP_LENGTH_LOW			0x07
#define DNMXL_TEMP_LENGTH_HIGH			0x00

#define DNMXL_BYTE_LOW					0x01
#define DNMXL_BYTE_HIGH					0x00

#define DNMXL_TRQ_LOW					0x18
#define DNMXL_TRQ_HIGH					0x00
#define DNMXL_LED_LOW					0x19
#define DNMXL_LED_HIGH					0x00
#define DNMXL_GOAL_LOW  	       		0x1E
#define DNMXL_GOAL_HIGH					0x00
#define DNMXL_TEMP_LOW  	       		0x2E
#define DNMXL_TEMP_HIGH					0x00


#define OFF                         0
#define ON                          1
#define LEFT						0
#define RIGTH                       1
#define AX_BYTE_READ                1
#define AX_BYTE_READ_POS            2
#define AX_RESET_LENGTH				2
#define AX_ACTION_LENGTH			2
#define AX_ID_LENGTH                4
#define AX_LR_LENGTH                4
#define AX_SRL_LENGTH               4
#define AX_RDT_LENGTH               4
#define AX_LEDALARM_LENGTH          4
#define AX_SALARM_LENGTH            4
#define AX_TL_LENGTH                4
#define AX_VL_LENGTH                6
#define AX_CM_LENGTH                6
#define AX_CS_LENGTH                6
#define AX_CCW_CW_LENGTH            8
#define AX_BD_LENGTH                4
#define AX_TEM_LENGTH               4
#define AX_REG_LENGTH               4
#define AX_MOVING_LENGTH            4
#define AX_RWS_LENGTH               4
#define AX_VOLT_LENGTH              4
#define AX_LED_LENGTH               4
#define AX_TORQUE_LENGTH            4
#define AX_POS_LENGTH               4
#define AX_GOAL_LENGTH              5
#define AX_MT_LENGTH                5
#define AX_PUNCH_LENGTH             5
#define AX_SPEED_LENGTH             5
#define AX_GOAL_SP_LENGTH           7
#define AX_ACTION_CHECKSUM			250
#define BROADCAST_ID                254
#define AX_START                    255
#define AX_CCW_AL_L                 255
#define AX_CCW_AL_H                 3
#define TIME_OUT                    16
#define Tx_MODE                     1
#define Rx_MODE                     0
#define LOCK                        1

	void switchCom(unsigned char DirPin,unsigned char Mode);

	int read_error(void);
	int readRegister(unsigned char numBytes);

	void Dynamixelinit(unsigned int baudrate);

	int Dynamixelreset(unsigned char ID);
	int Dynamixelping(unsigned char ID, unsigned char Dprotocol);

	int DynamixelsetID(unsigned char ID, unsigned char newID);
	int DynamixelsetBD(unsigned char ID, long baud);

	int Dynamixelmove(unsigned char ID, int Position, unsigned char Dynamixelprotocol);
	int DynamixelmoveSpeed(unsigned char ID, int Position, int Speed, unsigned char Dynamixelprotocol);

	int DynamixelsetEndless(unsigned char ID,bool Status);
	int Dynamixelturn(unsigned char ID, bool SIDE, int Speed);
	int DynamixelmoveRW(unsigned char ID, int Position);
	int DynamixelmoveSpeedRW(unsigned char ID, int Position, int Speed);

	void Dynamixelaction(void);

	int DynamixelsetTempLimit(unsigned char ID, unsigned char Temperature);
	int DynamixelsetAngleLimit(unsigned char ID, int CWLimit, int CCWLimit);
	int DynamixelsetVoltageLimit(unsigned char ID, unsigned char DVoltage, unsigned char UVoltage);
	int DynamixelsetMaxTorque(unsigned char ID, int MaxTorque);
	int DynamixelsetSRL(unsigned char ID, unsigned char SRL);
	int DynamixelsetRDT(unsigned char ID, unsigned char RDT);
	int DynamixelsetLEDAlarm(unsigned char ID, unsigned char LEDAlarm);
	int DynamixelsetShutdownAlarm(unsigned char ID, unsigned char SALARM);
	int DynamixelsetCMargin(unsigned char ID, unsigned char CWCMargin, unsigned char CCWCMargin);
	int DynamixelsetCSlope(unsigned char ID, unsigned char CWCSlope, unsigned char CCWCSlope);
	int DynamixelsetPunch(unsigned char ID, int Punch);

	int Dynamixelmoving(unsigned char ID);
	int DynamixellockRegister(unsigned char ID);
	int DynamixelRWStatus(unsigned char ID);

	int DynamixelreadRegister(unsigned char ID, unsigned char Register, unsigned char bytes);

	int DynamixelreadTemperature(unsigned char ID, unsigned char Dynamixelprotocol);

	int DynamixelreadTemperatureLimit(unsigned char ID);
	int DynamixelreadVoltage(unsigned char ID);
	int DynamixelreadMinVoltage(unsigned char ID);
	int DynamixelreadMaxVoltage(unsigned char ID);
	int DynamixelreadMaxTorque(unsigned char ID);
	int DynamixelreadPosition(unsigned char ID);
	int DynamixelreadCWAngle(unsigned char ID);
	int DynamixelreadCCWAngle(unsigned char ID);
	int DynamixelreadSpeed(unsigned char ID);
	int DynamixelreadLoad(unsigned char ID);



	int DynamixeltorqueStatus(unsigned char ID, bool Status,unsigned char DynamixelProtocol);
	int DynamixelledStatus(unsigned char ID, unsigned char LEDColor, unsigned char Dynamixelprotocol);

#endif
