/*
 * Este archivo contiene las funciones del driver I2C
 *
 * Josue Alejandro Gutierrez Alvarez
 */

#include "I2C.h"

/****************************************************************************************
*
*****************************************************************************************/
bool i2c_Init(uint8_t mult, uint8_t icr)
{

	SIM->SCGC5 |= SIM_SCGC5_PORTD(1);      //Turn on clock to A module
	PORTD->PCR[2] |= PORT_PCR_MUX(7) | PORT_PCR_ODE(1);    //Set PTD2 to mux 7 [I2C_SCL] Opendrain enable
	PORTD->PCR[3] |= PORT_PCR_MUX(7) | PORT_PCR_ODE(1);    //Set PTD3 to mux 7 [I2C_SDA] Opendrain enable

	SIM->SCGC4 |= SIM_SCGC4_I2C0(1);

	I2C0->F = I2C_F_ICR(icr) | I2C_F_MULT(mult);
	I2C0->C1 = I2C_C1_IICEN_MASK; // I2C Enable
	return true;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_DisableAck(void)
{
	I2C0->C1 |= I2C_C1_TXAK_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_EnableAck(void)
{
	I2C0->C1 &= ~I2C_C1_TXAK_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_RepeatedStart(void)
{
	I2C0->C1 |= I2C_C1_RSTA_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_EnterRxMode(void)
{
	I2C0->C1 &= ~I2C_C1_TX_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_Start(void)
{
	I2C0->C1 |= I2C_C1_TX_MASK;
	I2C0->C1 |= I2C_C1_MST_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_Stop(void)
{
	I2C0->C1 &= ~I2C_C1_MST_MASK;
	I2C0->C1 &= ~I2C_C1_TX_MASK;

	delay(80);
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_Wait(void)
{
	uint32_t i = 0;
	i = 1000000;

	while(((I2C0->S & I2C_S_IICIF_MASK) == 0) && i)
	{
		i--;
	}
	I2C0->S |= I2C_S_IICIF_MASK;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_WriteByte(uint8_t data)
{
	I2C0->D = (uint8_t)data;
	i2c_Wait();
}
/****************************************************************************************
*
*****************************************************************************************/
uint8_t i2c_ReadByte(void)
{
	return (uint8_t)( I2C0->D );
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_WriteRegister(uint8_t SlaveAddress, uint8_t RegisterAddress, uint8_t data)
{
	i2c_Start();
	i2c_WriteByte(((SlaveAddress << 1) | I2C_WRITE));
	i2c_WriteByte(RegisterAddress);
	i2c_WriteByte(data);
	i2c_Stop();

}
/****************************************************************************************
*
*****************************************************************************************/
uint8_t i2c_ReadRegister(uint8_t SlaveAddress,uint8_t RegisterAddress)
{
	uint8_t res = 0;

	i2c_Start();
	i2c_WriteByte(((SlaveAddress << 1) | I2C_WRITE));

	i2c_WriteByte(RegisterAddress);

	i2c_RepeatedStart();
	i2c_WriteByte(((SlaveAddress << 1) | I2C_READ));

	i2c_EnterRxMode();
	i2c_DisableAck();
	res = i2c_ReadByte();
	i2c_Wait();
	i2c_Stop();
	res = i2c_ReadByte();

	return res;
}
/****************************************************************************************
*
*****************************************************************************************/
uint8_t i2c_ReadMultRegister(uint8_t SlaveAddress,
                             uint8_t RegisterAddress,uint8_t n_data, uint8_t *res)
{
	uint8_t i = 0;

	i2c_Start();
	i2c_WriteByte(((SlaveAddress << 1) | I2C_WRITE));
	i2c_Wait();

	i2c_WriteByte(RegisterAddress);
	i2c_Wait();

	i2c_RepeatedStart();

	i2c_WriteByte(((SlaveAddress << 1) | I2C_READ));
	i2c_Wait();

	i2c_EnterRxMode();
	i2c_EnableAck();

	i = i2c_ReadByte();
	i2c_Wait();

	for(i=0;i<(n_data-2);i++)
	{
		*res = i2c_ReadByte();
		res++;
		i2c_Wait();
	}

	i2c_DisableAck();

	*res = i2c_ReadByte();
	res++;
	i2c_Wait();

	i2c_Stop();

	*res = i2c_ReadByte();

	delay(50);
	return 0;
}
/****************************************************************************************
*
*****************************************************************************************/
void i2c_WriteMultRegister(uint8_t SlaveAddress,
                           uint8_t RegisterAddress, uint8_t n_data, uint8_t *data)
{
	uint8_t i = 0;

	i2c_Start();
	i2c_WriteByte(((SlaveAddress << 1) | I2C_WRITE));
	i2c_Wait();

	i2c_WriteByte(RegisterAddress);
	i2c_Wait();

	for(i=0;i<n_data;i++)
	{
		i2c_WriteByte(*data);
		i2c_Wait();
		data++;
	}
	i2c_Stop();

	delay(50);
}
/****************************************************************************************
*
*****************************************************************************************/
void delay(uint32_t t)
{
	uint32_t cnt = 0;
	for(cnt=0; cnt<t; cnt++)
	{
		__asm("nop");
	};
}
/*****************************************************************************************/
