/*
 * Este archivo contiene las funciones del driver LCD
 *
 * Josue Alejandro Gutierrez Alvarez
 */

#include "ST7789.h"
#include "Fonts.h"

#define	PORTRST		PORTB
#define GPIORST		GPIOB
#define RST			4

#define	PORTDC		PORTB
#define GPIODC		GPIOB
#define DC			5

#define	PORTBLK		PORTA
#define GPIOBLK		GPIOA
#define BLK			17


void SPIInit(void){

	SIM->SCGC6 |= SIM_SCGC6_SPI0(1);												// Clock gate enable for SPI1
	SPI0->MCR = SPI_MCR_HALT(1) | SPI_MCR_PCSIS(0x3F) | SPI_MCR_CONT_SCKE(1);		// SPI0_HALT(Stop SPI module), SPI0_PCSIS(Chip select inactive state 1 . Inactive high)
	SPI0->CTAR[0] =  SPI_CTAR_DBR(1) | SPI_CTAR_FMSZ(7) | SPI_CTAR_PBR(0) | SPI_CTAR_BR(0) | SPI_CTAR_CPOL(1) | SPI_CTAR_CPHA(1); // SPI MODE 3
	SPI0->CTAR[1] =  SPI_CTAR_DBR(1) | SPI_CTAR_FMSZ(15) | SPI_CTAR_PBR(0) | SPI_CTAR_BR(0) | SPI_CTAR_CPOL(1) | SPI_CTAR_CPHA(1); // SPI MODE 3
	SPI0->MCR = SPI_MCR_MSTR(1) | SPI_MCR_PCSIS(0x3F) | SPI_MCR_CLR_TXF(1) | SPI_MCR_CLR_RXF(1);		// Set SPI0 as Master and Clear TX an RX buffers
	SPI0->TCR = 0;																						// SPI transfer counter to 0

	SIM->SCGC5 |= SIM_SCGC5_PORTD(1);		// Enable clock for PORTD
	SIM->SCGC5 |= SIM_SCGC5_PORTA(1);		// Enable clock for PORTA
	SIM->SCGC5 |= SIM_SCGC5_PORTB(1);		// Enable clock for PORTB

	PORTD->PCR[0] = PORT_PCR_MUX(1);		// SPI1->CS ALT2
	PORTA->PCR[15] = PORT_PCR_MUX(2);		// SPI1->CLK ALT2
	PORTA->PCR[16] = PORT_PCR_MUX(2);		// SPI1->MOSI ALT2
	//PORTA->PCR[17] = PORT_PCR_MUX(1);		// SPI1->MISO ALT2

	PORTRST->PCR[RST] = PORT_PCR_MUX(1);		// Reset Pin PORTB4
	GPIORST->PDDR |= (1<<RST);					// Output

	PORTDC->PCR[DC] = PORT_PCR_MUX(1);		// PIN as GPIO  Data/Command Pin PORTD2
	GPIODC->PDDR |= (1<<DC);					// Output

	PORTBLK->PCR[BLK] = PORT_PCR_MUX(1);		// Backlight Pin PORTA17
	GPIOBLK->PDDR |= (1<<BLK);					// Output

}

void SPITransfer(uint8_t dataByte){
	SPI0->PUSHR = dataByte | SPI_PUSHR_CTAS(0);
}

void writeCommand(uint8_t command){
	GPIODC->PCOR |= (1<<DC);	// Set to command mode
	SPITransfer(command);
	delayus(1);
	GPIODC->PSOR |= (1<<DC);	// Set data mode
}

void writeData(uint8_t data){
	GPIODC->PSOR |= (1<<DC);	// Set data mode
	SPITransfer(data);
	delayus(1);
}

void initDisplay(void){

	unsigned long indexInit;

	SPIInit();

	GPIORST->PCOR |= (1<<RST);	// Hardware reset
	delayms(10);
	GPIORST->PSOR |= (1<<RST);
	delayms(100);

	writeCommand(ST7789_SWRESET);	// Software reset
	delayms(150);					// Delay of 150ms
	writeCommand(ST7789_SLPOUT);	// Out of sleep mode
	delayms(200);					// Delay of 200ms
	writeCommand(ST7789_COLMOD);	// Set Color Mode
	writeData(ST7789_16BIT);		// 16-bit color
	delayms(10);					// Delay of 10ms
	writeCommand(ST7789_MADCTL);	// Memory access control
	writeData(ORIENTATION4);		// bottom to top refresh

	writeCommand(ST7789_CASET);		// Column address set
	writeData(ST7789_XSTART >> 8);
	writeData(ST7789_XSTART & 0xFF);
	writeData((ST7789_TFTWIDTH+ST7789_XSTART)>>8);
	writeData((ST7789_TFTWIDTH+ST7789_XSTART) & 0xFF);
	writeCommand(ST7789_RASET);
	writeData(ST7789_YSTART >> 8);
	writeData(ST7789_YSTART & 0xFF);
	writeData((ST7789_TFTHEIGHT+ST7789_YSTART)>>8);
	writeData((ST7789_TFTHEIGHT+ST7789_YSTART) & 0xFF);

	writeCommand(ST7789_INVOFF);	// Inversion OFF
	delayms(10);					// Delay of 10ms
	writeCommand(ST7789_NORON);		// Normal display ON
	delayms(10);					// Delay of 10ms
	writeCommand(ST7789_DISPON);	// DisplaY ON
	delayms(100);					// Delay of 100ms

    writeCommand(ST7789_RAMWR);

    for(indexInit = 0; indexInit != 320*240; indexInit++){
        	SPI0->PUSHR = 0x0000 | SPI_PUSHR_CTAS(1);
        	delayus(1);
     }

    GPIOBLK->PSOR |= (1<<BLK);			// Turn on backlight
}

void displayDrawPixel(uint8_t x, uint8_t y, uint16_t color){

	writeCommand(ST7789_CASET);
	writeData(y >> 8);
	writeData(y & 0xFF );
	writeData(y >> 8);
	writeData(y & 0xFF);
	writeCommand(ST7789_RASET);
	writeData(x >> 8);
	writeData(x & 0xFF );
	writeData(x >> 8);
	writeData(x & 0xFF);
	writeCommand(ST7789_RAMWR);

	SPI0->PUSHR =color | SPI_PUSHR_CTAS(1);
	delayus(1);


}

void displayPutChar(char c, uint16_t x, uint16_t y, uint8_t font, uint16_t fColor, uint16_t bColor){

	int i,j;
	unsigned int nCols;
	unsigned int nRows;
	unsigned int nBytes;
	uint32_t PixelRow;
	uint32_t Mask;
	unsigned char *pFont;
	unsigned char *pChar;
	unsigned char *FontTable[] = {(unsigned char *)MonoSpatial, (unsigned char *)MonoSpace,
	 (unsigned char *)Unispace};

	pFont = (unsigned char *)FontTable[font];

	nCols = *pFont;
	nRows = *(pFont + 1);
	nBytes = *(pFont +2);

	pChar = pFont + (nBytes * (c - 0x20)) + nBytes;// -1;

	writeCommand(ST7789_CASET);
	writeData(y >> 8);
	writeData(y & 0xFF );
	writeData((y + nCols - 1) >> 8);
	writeData((y + nCols - 1) & 0xFF);

	writeCommand(ST7789_RASET);
	writeData(x >> 8);
	writeData(x & 0xFF );
	writeData((x + nRows - 1) >> 8);
	writeData((x + nRows - 1) & 0xFF);

	writeCommand(ST7789_RAMWR);

	for (i = nRows - 1; i >= 0; i--) {

		if (nCols == 24){
			PixelRow = *(pChar) << 16;
			PixelRow += *(pChar+1) << 8;
			PixelRow += *(pChar+2);
			pChar++;
			pChar++;
			pChar++;
		}else if (nCols == 16){
			PixelRow = *(pChar) << 8;
			PixelRow += *(pChar+1);
			pChar++;
			pChar++;
		}else{
			PixelRow = *(pChar);
			pChar++;
		}

		Mask = 0x01 << (nCols-1); // 0x8000 for 16
		for (j = 0; j < nCols; j ++) {
			if ((PixelRow & Mask) == 0){
				writeData(fColor>>8);
				writeData(fColor & 0xFF);
			}else{
				writeData(bColor>>8);
				writeData(bColor & 0xFF);

			 }
			Mask = Mask >> 1;
		}
	}
}



void displayPutStr(char *pString, uint16_t x, uint16_t y, uint8_t Size, uint16_t fColor, uint16_t bColor) {

// loop until null-terminator is seen
while (*pString != 0x00) {
// draw the character
displayPutChar(*pString++, x, y, Size, fColor, bColor);
// advance the y position
if (Size == 1)
 y = y + 8;
else if (Size == 2)
 y = y + 24;
 else
 y = y + 8;

// bail out if y exceeds 241
if (y > 241) break;
 }
}

void displayPutImg(uint16_t y, uint16_t x, const uint16_t *Img, uint16_t width, uint16_t height){

	unsigned long indexImg;

	writeCommand(ST7789_CASET);
	writeData(y >> 8);
	writeData(y & 0xFF );
	writeData((y + width - 1) >> 8);
	writeData((y + width - 1) & 0xFF);

	writeCommand(ST7789_RASET);
	writeData(x >> 8);
	writeData(x & 0xFF );
	writeData((x + height -1) >> 8);
	writeData((x + height -1) & 0xFF);

	writeCommand(ST7789_RAMWR);

	for(indexImg = 0; indexImg != height*width; indexImg++){
	    	SPI0->PUSHR =Img[indexImg] | SPI_PUSHR_CTAS(1);
	    		delayus(1);
	    }
}

void displayAnimation(uint16_t y, uint16_t x, const uint16_t *Img, uint16_t width, uint16_t height, uint8_t frames){

	unsigned long indexImg;
	unsigned long indexAnimation = 0;

	writeCommand(ST7789_CASET);
	writeData(y >> 8);
	writeData(y & 0xFF );
	writeData((y + width - 1) >> 8);
	writeData((y + width - 1) & 0xFF);

	writeCommand(ST7789_RASET);
	writeData(x >> 8);
	writeData(x & 0xFF );
	writeData((x + height -1) >> 8);
	writeData((x + height -1) & 0xFF);

	writeCommand(ST7789_RAMWR);

	for(indexImg = 0; indexImg != height*width*frames; indexImg++){
		SPI0->PUSHR =Img[indexImg] | SPI_PUSHR_CTAS(1);
		delayus(1);
		indexAnimation++;
		if(indexAnimation == height*width ){
			delayms(4);
			indexAnimation = 0;
		}
	}
}

void displaydrawRectangle(uint16_t x,uint16_t y, uint16_t w, uint16_t h,uint16_t color){

	int i;

	writeCommand(ST7789_CASET);
	writeData(y >> 8);
	writeData(y & 0xFF );
	writeData((y + w - 1) >> 8);
	writeData((y + w - 1) & 0xFF);

	writeCommand(ST7789_RASET);
	writeData(x >> 8);
	writeData(x & 0xFF );
	writeData((x + h - 1) >> 8);
	writeData((x + h - 1) & 0xFF);

	writeCommand(ST7789_RAMWR);

	for(i = 0; i != h*w; i++){
		    	SPI0->PUSHR = color | SPI_PUSHR_CTAS(1);
		    		delayus(1);
		    }
}

void displaydrawCircle(){

}

