/*
 * Este archivo contiene las definiciones del driver LCD
 *
 * Josue Alejandro Gutierrez Alvarez
 */

#include <stdio.h>
#include "MK26F18.h"
#include "Delay.h"

#ifndef ST7789_H_
#define ST7789_H_

// Color definitions

#define BLACK 	0x0000
#define BLUE 	0x001F
#define RED 	0xF800
#define GREEN 	0x07E0
#define CYAN 	0x07FF
#define MAGENTA 0xF81F
#define YELLOW 	0xFFE0
#define ORANGE	0xFB20
#define WHITE 	0xFFFF
#define GRAY 	0x4A69
#define DARK 	0x18C3

#define ST7789_TFTWIDTH 	319
#define ST7789_TFTHEIGHT 	239

#define ST7789_XSTART 0
#define ST7789_YSTART 0
#define ST7789_16BIT   0x55

#define ST7789_NOP     0x00
#define ST7789_SWRESET 0x01
#define ST7789_RDDID   0x04
#define ST7789_RDDST   0x09

#define ST7789_SLPIN   0x10
#define ST7789_SLPOUT  0x11
#define ST7789_PTLON   0x12
#define ST7789_NORON   0x13

#define ST7789_INVOFF  0x20
#define ST7789_INVON   0x21
#define ST7789_DISPOFF 0x28
#define ST7789_DISPON  0x29
#define ST7789_CASET   0x2A
#define ST7789_RASET   0x2B
#define ST7789_RAMWR   0x2C
#define ST7789_RAMRD   0x2E
#define ST7789_RAMWRC  0x3C

#define ST7789_PTLAR   0x30
#define ST7789_COLMOD  0x3A
#define ST7789_MADCTL  0x36

#define ST7789_MADCTL_MY  0x80
#define ST7789_MADCTL_MX  0x40
#define ST7789_MADCTL_MV  0x20
#define ST7789_MADCTL_ML  0x10
#define ST7789_MADCTL_RGB 0x00

#define ST7789_RDID1   0xDA
#define ST7789_RDID2   0xDB
#define ST7789_RDID3   0xDC
#define ST7789_RDID4   0xDD

#define ORIENTATION1	0x00
#define ORIENTATION2	0x60
#define ORIENTATION3	0xA0
#define ORIENTATION4	0xC0

void SPITransfer(uint8_t dataByte);
void SPIInit(void);
void writeCommand(uint8_t command);
void writeData(uint8_t data);
void initDisplay(void);

void displayPutChar(char c, uint16_t x, uint16_t y, uint8_t font, uint16_t fColor, uint16_t bColor);
void displayPutStr(char *pString, uint16_t x, uint16_t y, uint8_t Size, uint16_t fColor, uint16_t bColor);


void displayDrawPixel(uint8_t x, uint8_t y, uint16_t color);

void displayPutImg(uint16_t x, uint16_t y, const uint16_t *Img, uint16_t width, uint16_t height);

void displayAnimation(uint16_t y, uint16_t x, const uint16_t *Img, uint16_t width, uint16_t height, uint8_t frames);

void displaydrawRectangle(uint16_t x,uint16_t y, uint16_t w, uint16_t h,uint16_t color);
void displaydrawCircle();


#endif /* ST7789_H_ */
