#include <stdio.h>
#include "pin_mux.h"
#include "clock_config.h"
#include "MK26F18.h"

#include "Delay.h"
#include "ST7789.h"
#include "Splashscreen.h"
#include "Interface.h"
#include "Animations.h"
#include "Dynamixel.h"
#include "ModelAX.h"
/* TODO: insert other include files here. */

/* TODO: insert other definitions and declarations here. */

#define PORTBTN1 	PORTE
#define PORTBTN2 	PORTE
#define PORTBTN3 	PORTE
#define PORTBTN4 	PORTE

#define GPIOBTN1 	GPIOE
#define GPIOBTN2 	GPIOE
#define GPIOBTN3 	GPIOE
#define GPIOBTN4 	GPIOE

#define BTN1 	25
#define BTN2 	24
#define BTN3 	9
#define BTN4 	8

#define BTIME	200

#define YP	0
#define	XP	1
#define YM	2
#define XM	3

#define BACKGROUND		DARK
#define dataBarColor	GRAY
#define dataColor		WHITE
#define dataColorError	RED

#define MINMAXVAL	156

const char Baudrates[8][10] = {"1,000,000","500,000  ","400,000  ","200,000  ","115,200  ","57,600   ","19,200   ","9,600     "};
const uint32_t configBaudrate[8] = {1000000,500000,400000,200000,115200,57600,19200,9600};

unsigned int currentBaudrate = 0;
unsigned int protocol = 1;

float motorRegisterDecimals = 0.0;
int currentRegister = 0;
int motorRegister = 0;
int currentMotor = 0;
int motorIndex = 0;
int motorList[255];


char motorRegisterDecimalsStr[4];
char motorRegisterBuffer[30];
char motorRegisterStr[8];

char statusBuffer[30];
char currentMotorStr[4];
char motorIndexStr[4];
char motorIDStr[4];

uint8_t CurrentScreen = 0;
uint8_t keyboardEnabled = 0;
uint8_t lockStatus = 1;
uint8_t motorID = 0;

void displayMainScreen();
void displayScanScreen();
void displayScanMotors();
void displayRegisterScreen();
void displayRestoreScreen(unsigned char option);
void displayConfigScreen();
void displayMotorStatus();

/** This is a test of a simple user interface for a hand-held motor configurator **/

int main(void) {

	BOARD_InitBootClocks();

	SIM->SCGC2 |= SIM_SCGC2_TPM1(1);				// Enables the TPM1 for delay functions
	SIM->SOPT2 |= SIM_SOPT2_TPMSRC(0x01);
	TPM1->SC = TPM_SC_CMOD(0x00);
	TPM1->SC |= TPM_SC_PS(0x02);

	SIM->SCGC5 |= SIM_SCGC5_PORTA(1);				// PORTA Clock enable
	SIM->SCGC5 |= SIM_SCGC5_PORTB(1);				// PORTB Clock enable
	SIM->SCGC5 |= SIM_SCGC5_PORTD(1);				// PORTD Clock enable
	SIM->SCGC5 |= SIM_SCGC5_PORTE(1);				// PORTE Clock enable

	PORTBTN1->PCR[BTN1] |= PORT_PCR_MUX(1);			// Pin as GPIO for Button 1
	PORTBTN2->PCR[BTN2] |= PORT_PCR_MUX(1);			// Pin as GPIO for Button 2
	PORTBTN3->PCR[BTN3] |= PORT_PCR_MUX(1);			// Pin as GPIO for Button 3
	PORTBTN4->PCR[BTN4] |= PORT_PCR_MUX(1);			// Pin as GPIO for Button 4

	GPIOBTN1->PDDR &= ~(1<<BTN1);					// Pin as Input for Button 1
	GPIOBTN2->PDDR &= ~(1<<BTN2);					// Pin as Input for Button 2
	GPIOBTN3->PDDR &= ~(1<<BTN3);					// Pin as Input for Button 3
	GPIOBTN4->PDDR &= ~(1<<BTN4);					// Pin as Input for Button 4

	Dynamixelinit(configBaudrate[currentBaudrate]);			// Start motor communications
	initDisplay();											// Initialization for TFT Display ST7789V

	displayPutImg(0,0,StartScreen,240,320);					// Display SplashScreen
	delayms(3000);											// Wait for SplashScreen

	displayMainScreen();									// Display main menu screen

	while(1){


		if( (CurrentScreen == 1) & (motorIndex > 0)){		// Scan/Select Motor Screen

			if(Dynamixelping(motorID,protocol) != -128){						// Update motor status
				displaydrawRectangle(48,2,6,6,GREEN);				// Online Motor
			}else{
				displaydrawRectangle(48,2,6,6,RED);					// Off-line Motor
			}

			motorRegister = DynamixelreadPosition(motorID);				// Read motor position
			if((motorRegister < -127) | (motorRegister > 2048)){
				displayPutStr("---*  ",98-8,10,2,dataColor,dataBarColor);
			}else {
				motorRegister = (motorRegister*300)/1023;
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"*  ");
				displayPutStr(motorRegisterBuffer,98-8,10,2,dataColor,dataBarColor);
			}

			motorRegister = DynamixelreadCWAngle(motorID);			// Read motor CW Angle Limit
			if((motorRegister < -127) | (motorRegister > 2048)){
				displayPutStr("MIN",88,MINMAXVAL,0,dataColor,dataBarColor);displayPutStr("ERROR ",88,MINMAXVAL+32,0,dataColorError,dataBarColor);
			} else {
				motorRegister = (motorRegister*300)/1023;
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,"MIN ");strcat(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"*   ");
				displayPutStr(motorRegisterBuffer,88,MINMAXVAL,0,dataColor,dataBarColor);
			}

			motorRegister = DynamixelreadCCWAngle(motorID);		// Read motor CCW Angle Limit
			if((motorRegister < -127) | (motorRegister > 2048)){
				displayPutStr("MAX",108,MINMAXVAL,0,dataColor,dataBarColor);displayPutStr("ERROR ",108,MINMAXVAL+32,0,dataColorError,dataBarColor);
			} else {
				motorRegister = (motorRegister*300)/1023;
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,"MAX ");strcat(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"*   ");
				displayPutStr(motorRegisterBuffer,108,MINMAXVAL,0,dataColor,dataBarColor);
			}


			motorRegister = DynamixelreadVoltage(motorID);				// Read motor voltage
			if((motorRegister < -127) | (motorRegister > 999)){
				displayPutStr("-.-v ",170-8,10,2,dataColor,dataBarColor);
			}else {
				motorRegisterDecimals = motorRegister - ((motorRegister/10)*10);
				itoa(motorRegisterDecimals,motorRegisterDecimalsStr,10);
				itoa((motorRegister/10),motorRegisterStr,10);
				strcpy(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,".");
				strcat(motorRegisterBuffer,motorRegisterDecimalsStr);strcat(motorRegisterBuffer,"v ");
				displayPutStr(motorRegisterBuffer,170-8,10,2,dataColor,dataBarColor);
			}

			motorRegister = DynamixelreadMinVoltage(motorID);			// Read motor minimum voltage limit
			if((motorRegister < -127) | (motorRegister > 999)){
				displayPutStr("MIN",160,MINMAXVAL,0,dataColor,dataBarColor);displayPutStr("ERROR ",160,MINMAXVAL+32,0,dataColorError,dataBarColor);
			} else {
				motorRegisterDecimals = motorRegister - ((motorRegister/10)*10);
				itoa(motorRegisterDecimals,motorRegisterDecimalsStr,10);
				itoa(motorRegister/10,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,"MIN ");strcat(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,".");
				strcat(motorRegisterBuffer,motorRegisterDecimalsStr);strcat(motorRegisterBuffer,"v ");
				displayPutStr(motorRegisterBuffer,160,MINMAXVAL,0,dataColor,dataBarColor);

			}

			motorRegister = DynamixelreadMaxVoltage(motorID);			// Read motor maximum voltage limit
			if((motorRegister < -127) | (motorRegister > 999)){
				displayPutStr("MAX",180,MINMAXVAL,0,dataColor,dataBarColor);displayPutStr("ERROR ",180,MINMAXVAL+32,0,dataColorError,dataBarColor);
			} else {
				motorRegisterDecimals = motorRegister - ((motorRegister/10)*10);
				itoa(motorRegisterDecimals,motorRegisterDecimalsStr,10);
				itoa(motorRegister/10,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,"MAX ");strcat(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,".");
				strcat(motorRegisterBuffer,motorRegisterDecimalsStr);strcat(motorRegisterBuffer,"v ");
				displayPutStr(motorRegisterBuffer,180,MINMAXVAL,0,dataColor,dataBarColor);
			}


			motorRegister = DynamixelreadTemperature(motorID,protocol);		// Read motor temperature
			if((motorRegister < -127) | (motorRegister > 999)){
				displayPutStr("--*C ",242-8,10,2,dataColor,dataBarColor);
			}else {
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"*C  ");
				displayPutStr(motorRegisterBuffer,242-8,10,2,dataColor,dataBarColor); //242
			}

			motorRegister = DynamixelreadTemperatureLimit(motorID);		// Read motor MAX Temperature Limit
			if((motorRegister < -127) | (motorRegister > 999)){
				displayPutStr("MAX",252,MINMAXVAL,0,dataColor,dataBarColor);displayPutStr("ERROR ",252,MINMAXVAL+32,0,dataColorError,dataBarColor);
			} else {
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,"MAX ");strcat(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"*C  ");
				displayPutStr("MIN --*C ",232,MINMAXVAL,0,dataColor,dataBarColor);
				displayPutStr(motorRegisterBuffer,252,MINMAXVAL,0,dataColor,dataBarColor);
			}


		}

		if( (CurrentScreen == 2) & (motorIndex > 0)){				// Configure Motor Screen

			if(Dynamixelping(motorID,protocol) != -128){						// Update motor status
				displaydrawRectangle(48,2,6,6,GREEN);				// Online Motor
			}else{
				displaydrawRectangle(48,2,6,6,RED);					// Off-line Motor
			}

			displayPutStr(AXRegisterDescription[currentRegister],72,4,0,dataColor,dataBarColor);									// Display register description

			motorRegister = DynamixelreadRegister(motorID,AXRegisterAddress[currentRegister],AXRegisterBytes[currentRegister]);		// Read motor register
			if((motorRegister < -127) | (motorRegister > 9999)){
				displayPutStr("ERROR",90,4,2,dataColorError,dataBarColor);
			} else {
				itoa(motorRegister,motorRegisterStr,10);
				strcpy(motorRegisterBuffer,motorRegisterStr);strcat(motorRegisterBuffer,"     ");
				displayPutStr(motorRegisterBuffer,90,4,2,dataColor,dataBarColor);													// Display Register Value
			}

			displayPutStr(AXRegisterRange[currentRegister],136,4,0,dataColor,dataBarColor);											// Display register range

			//displayPutStr(" Over Heating Error! ",252,34,0,dataColorError,dataBarColor);
			//displaydrawRectangle(207,139,10,14,RED);

		}


		if( (CurrentScreen == 3) & (motorIndex > 0)){				// Reset Motor Screen

			if(Dynamixelping(motorID,protocol) != -128){						// Update motor status
				displaydrawRectangle(48,2,6,6,GREEN);				// Online Motor
			}else{
				displaydrawRectangle(48,2,6,6,RED);					// Off-line Motor
			}
		}

		if (CurrentScreen == 4){

			displayPutStr(Baudrates[currentBaudrate],112,8,2,dataColor,dataBarColor);

			if( protocol == 1){
				displayPutStr("1.0",188,8,2,dataColor,dataBarColor);
			}else if(protocol == 2){
				displayPutStr("2.0",188,8,2,dataColor,dataBarColor);
			}

		}




		if((GPIOBTN1->PDIR & (1<<BTN1))){
			while((GPIOBTN1->PDIR & (1<<BTN1)));
			delayms(BTIME);

			switch(CurrentScreen){
			case 0:
				CurrentScreen = 1;					// Main Menu to Register Screen
				displayScanScreen();
				displayScanMotors();				// Start motor search
				break;
			case 1:
				displayScanMotors();				// Start motor search
				break;

			case 2:

				break;

			case 3:
				if(motorIndex > 0){
					Dynamixelreset(motorID);
					displayPutStr("Scan to update motor list.",44,12,0,YELLOW,BACKGROUND);
					displayRestoreScreen(2);
				}else{
					displayPutStr("Scan and select motor.    ",44,12,0,YELLOW,BACKGROUND);
				}
				break;
			case 4:
				currentBaudrate++;					// Roll-out Baud Rate
				if(currentBaudrate > 7)
					currentBaudrate = 0;
				Dynamixelinit(configBaudrate[currentBaudrate]);	// DynamixelInit different Baud rate
				break;
			}
		}

		if((GPIOBTN2->PDIR & (1<<BTN2))){
			while((GPIOBTN2->PDIR & (1<<BTN2)));
			delayms(BTIME);

			switch(CurrentScreen){
			case 0:
				CurrentScreen = 2;					// Main Menu to Register Screen
				displayRegisterScreen();			// Display Register Screen
				displayMotorStatus();				// Display motor status
				break;
			case 1:
				displayMotorStatus();				// Display motor status
				if(currentMotor > 0)
					currentMotor--;						// Go to previous motor
				motorID = motorList[currentMotor];  // Update selected motor
				displayMotorStatus();				// Display motor status
				break;

			case 2:
				displayMotorStatus();				// Display motor status
				if(currentRegister > 0)
					currentRegister--;				// Go to previous register
				break;

			case 3:
				displayMotorStatus();				// Display motor status
				if(currentMotor > 0)
					currentMotor--;						// Go to previous motor
				motorID = motorList[currentMotor];  // Update selected motor
				displayMotorStatus();				// Display motor status
				break;
			case 4:
				protocol++;
				if(protocol > 2)
					protocol = 1;
				break;
			}

		}

		if((GPIOBTN3->PDIR & (1<<BTN3))){
			while((GPIOBTN3->PDIR & (1<<BTN3)));
			delayms(BTIME);

			switch(CurrentScreen){
			case 0:
				CurrentScreen = 3;					// Main Menu to Register Screen
				displayRestoreScreen(1);				// Display Register Screen
				displayMotorStatus();				// Display motor status
				break;
			case 1:
				displayMotorStatus();				// Display motor status
				if(currentMotor < (motorIndex - 1))
					currentMotor++;						// Go to next motor
				motorID = motorList[currentMotor];  // Update selected motor
				displayMotorStatus();				// Display motor status
				break;

			case 2:
				displayMotorStatus();				// Display motor status
				if(currentRegister < 34)
					currentRegister++;				// Go to next register
				break;

			case 3:
				displayMotorStatus();				// Display motor status
				if(currentMotor < (motorIndex - 1))
					currentMotor++;						// Go to next motor
				motorID = motorList[currentMotor];	// Update selected motor
				displayMotorStatus();				// Display motor status
				break;
			case 4:
				// DO TOUCH CALIBRATION
				break;
			}
		}

		if((GPIOBTN4->PDIR & (1<<BTN4))){
			while((GPIOBTN4->PDIR & (1<<BTN4)));
			delayms(BTIME);

			switch(CurrentScreen){
			case 0:
				CurrentScreen = 4;					// Main Menu to Register Screen
				displayConfigScreen();				// Display Configuration Screen
				break;
			case 1:
				CurrentScreen = 0;
				displayMainScreen();				// Display Main menu
				DynamixelledStatus(BROADCAST_ID,0,protocol);  		// Turn off all motors LED
				break;

			case 2:
				if(!keyboardEnabled){
					CurrentScreen = 0;
					displayMainScreen();				// Display Main menu
					DynamixelledStatus(BROADCAST_ID,0,protocol);  		// Turn off all motors LED
				} else {
					CurrentScreen = 2;					// Main Menu to Register Screen
					displayRegisterScreen();			// Display Register Screen
					displayMotorStatus();				// Display motor status
				}
				break;

			case 3:
				CurrentScreen = 0;
				displayMainScreen();				// Display Main menu
				DynamixelledStatus(BROADCAST_ID,0,protocol);  		// Turn off all motors LED
				break;
			case 4:
				CurrentScreen = 0;
				displayMainScreen();				// Display Main menu
				DynamixelledStatus(BROADCAST_ID,0,protocol);  		// Turn off all motors LED
				break;
			}


		}

	}


	return 0 ;
}

void displayMainScreen(){

	displaydrawRectangle(0,0,240,320,BACKGROUND);			// Clear screen and set background
	displayPutImg(0,0,mainMenuTittle,240,32);				// Display Main menu title
	displayPutImg(0,312,mainButtons,240,8);					// Display Main menu buttons

	displayAnimation(10,64,scanButton,100,100,6);			// Display Scan Button animation
	displayAnimation(130,64,registersButton,100,100,6);		// Display Register Button animation
	displayAnimation(10,184,resetButton,100,100,6);			// Display Reset Button animation
	displayAnimation(130,184,configButton,100,100,6);		// Display Config Button animation
}

void displayScanScreen(){

	displaydrawRectangle(0,0,240,320,BACKGROUND);		// Clear screen
	displayPutImg(0,0,scanSelect,240,32);				// Screen title Scan/Select Motors

	displaydrawRectangle(64,0,240,4,BACKGROUND);		// Clear Progress Bar

	displaydrawRectangle(68,0,240,64+6,dataBarColor);
	displayPutStr("Position",72,4,0,dataColor,dataBarColor);
	displayPutStr("---*",90,10,2,dataColor,dataBarColor);
	displayPutStr("MIN ---*",88,MINMAXVAL,0,dataColor,dataBarColor);
	displayPutStr("MAX ---*",108,MINMAXVAL,0,dataColor,dataBarColor);

	displaydrawRectangle(140,0,240,64+6,dataBarColor);
	displayPutStr("Voltage",144,4,0,dataColor,dataBarColor);
	displayPutStr("-.-v  ",162,10,2,dataColor,dataBarColor);
	displayPutStr("MIN -.-v",160,MINMAXVAL,0,dataColor,dataBarColor);
	displayPutStr("MAX -.-v",180,MINMAXVAL,0,dataColor,dataBarColor);

	displaydrawRectangle(212,0,240,64+6,dataBarColor);
	displayPutStr("Temperature",216,4,0,dataColor,dataBarColor);
	displayPutStr("--*C",234,10,2,dataColor,dataBarColor);
	displayPutStr("MIN --*C",232,MINMAXVAL,0,dataColor,dataBarColor);
	displayPutStr("MAX --*C",252,MINMAXVAL,0,dataColor,dataBarColor);

	displayPutImg(0,288,mainMenu,240,32);			// scan/select Menu
}

void displayRegisterScreen(){

	displaydrawRectangle(0,0,240,320,BACKGROUND);		// Clear screen
	displayPutImg(0,0,configRegisters,240,32);			// Screen title Configure Register

	lockStatus = DynamixelreadRegister(motorID,24,1);
	if(lockStatus == 0){
		displayPutImg(216,38,unlocked,22,22);					// unlocked position
	}else{
		displayPutImg(216,38,locked,22,22);					// locked position
	}

	displaydrawRectangle(68,0,240,86,dataBarColor);	// Register data and description bar
	displayPutStr("Add: ---, Motor register     ",72,4,0,dataColor,dataBarColor);
	displayPutStr("---   ",90,4,2,dataColor,dataBarColor);
	displayPutStr("Range: --- to ---.           ",136,4,0,dataColor,dataBarColor);

	displayPutImg(0,158,keyboard,240,126);					// Display keyboard

	displayPutImg(0,288,regMenu,240,32);					// configure register menu
}

void displayRestoreScreen(unsigned char option){

	if(option == 1){

		displaydrawRectangle(0,0,240,320,BACKGROUND);		// Clear screen
		displayPutImg(0,0,factoryReset,240,32);				// Screen title Restore default

		displaydrawRectangle(68,0,240,212,dataBarColor);
		displayPutStr("Restore motor configurations",72,4,0,dataColor,dataBarColor);
		displayPutStr("to ",72+14,4,0,dataColor,dataBarColor);
		displayPutStr("Factory Default",72+14,4+24,0,ORANGE,dataBarColor);
		displayPutStr("?",72+14,4+(8*18),0,dataColor,dataBarColor);

		displayPutStr("/// Warning /////////////////",116,4,0,WHITE,RED);
		displayPutStr("This will set the ID to 1    ",136,4,0,dataColor,dataBarColor);
		displayPutStr("and Baud Rate to 1Mbps.      ",150,4,0,dataColor,dataBarColor);

		displayPutStr("Please avoid using identical ",176,4,0,dataColor,dataBarColor);
		displayPutStr("ID for multiple Motors.      ",192,4,0,dataColor,dataBarColor);
		displayPutStr("You may face communication   ",206,4,0,dataColor,dataBarColor);
		displayPutStr("failure or may not be able to",220,4,0,dataColor,dataBarColor);
		displayPutStr("detect Motors with same ID.  ",234,4,0,dataColor,dataBarColor);
		displayPutStr("/////////////////////////////",256,4,0,WHITE,RED);


		displayPutImg(0,288,resetMenu,240,32);
	}else if (option == 2){

		displaydrawRectangle(68,0,240,212,dataBarColor);
		displayPutStr("Restore motor configurations",72,4,0,dataColor,dataBarColor);
		displayPutStr("to ",72+14,4,0,dataColor,dataBarColor);
		displayPutStr("Factory Default",72+14,4+24,0,ORANGE,dataBarColor);
		displayPutStr("?",72+14,4+(8*18),0,dataColor,dataBarColor);
		displayPutImg(58,120,resetOK,124,124);
		displayPutStr("          Reset OK!         ",252,4,0,dataColor,dataBarColor);
	}
}


void displayConfigScreen(){

	displaydrawRectangle(0,0,240,320,BACKGROUND);
	displayPutImg(0,0,configuration,240,32);

	displayPutImg(0,32,smartMotorLogo,240,60);

	displaydrawRectangle(94,0,240,70,dataBarColor);
	displaydrawRectangle(94,0,2,70,BLUE);
	displayPutStr("Baud Rate (bps)",98,8,0,dataColor,dataBarColor);

	displaydrawRectangle(170,0,240,70,dataBarColor);
	displaydrawRectangle(170,0,2,70,YELLOW);
	displayPutStr("Protocol",174,8,0,dataColor,dataBarColor);

	displaydrawRectangle(248,0,240,32,dataBarColor);
	displaydrawRectangle(248,0,2,32,ORANGE);
	displayPutStr("TouchScreen Calibration",256,8,0,dataColor,dataBarColor);

	displayPutImg(0,288,configMenu,240,32);
}

void displayMotorStatus(){
	if(motorIndex > 0){
		displaydrawRectangle(48,2,6,6,GREEN);				// Online Motor
		itoa((currentMotor+1),currentMotorStr,10);
		itoa(motorIndex,motorIndexStr,10);
		itoa(motorID,motorIDStr,10);

		strcpy(statusBuffer,"Motor ");strcat(statusBuffer,currentMotorStr);
		strcat(statusBuffer," of ");strcat(statusBuffer,motorIndexStr);
		strcat(statusBuffer,", ID: ");strcat(statusBuffer,motorIDStr);
		displayPutStr(statusBuffer,44,12,0,WHITE,BACKGROUND);
	}else{
		displaydrawRectangle(48,2,6,6,RED);				// Online Motor
		displayPutStr("No motors detected! ",44,12,0,RED,BACKGROUND);
	}
}


void displayScanMotors(){

	displaydrawRectangle(38,0,200,20,BACKGROUND);					// Clear status bar
	displaydrawRectangle(64,0,240,4,BACKGROUND);					// Clear search bar

	displaydrawRectangle(48,2,6,6,BLUE);							// Search Motor
	displayPutStr("Scanning motors...    ",44,12,0,WHITE,BACKGROUND);

	motorIndex = 0;
	currentMotor = 0;

	for(motorID = 0; motorID < 254; motorID++){
		motorList[motorID] = 0;
	}

	for(motorID = 0; motorID < 254; motorID++){
		if(!(Dynamixelping(motorID,protocol) == -128) ){
			motorList[motorIndex++] = motorID;
		}
		displaydrawRectangle(64,0,((motorID*240)/253),4,GREEN);		// Progress Bar
	}

	displaydrawRectangle(38,0,200,20,BACKGROUND);					// Clear status bar
	displaydrawRectangle(64,0,240,4,BACKGROUND);					// Clear search bar

	if(motorIndex > 0){
		motorID = motorList[currentMotor];

		displaydrawRectangle(48,2,6,6,GREEN);				// Online Motor

		itoa((currentMotor+1),currentMotorStr,10);
		itoa(motorIndex,motorIndexStr,10);
		itoa(motorID,motorIDStr,10);

		strcpy(statusBuffer,"Motor ");strcat(statusBuffer,currentMotorStr);
		strcat(statusBuffer," of ");strcat(statusBuffer,motorIndexStr);
		strcat(statusBuffer,", ID: ");strcat(statusBuffer,motorIDStr);
		displayPutStr(statusBuffer,44,12,0,WHITE,BACKGROUND);

		DynamixelledStatus(BROADCAST_ID,0,protocol);  		// Turn off all motors LED
		DynamixelledStatus(motorID,1,protocol);	 			// Turn on the current motor LED
		DynamixeltorqueStatus(motorID,0,protocol);			// Unlock motor
	}else{
		displaydrawRectangle(48,2,6,6,RED);				// Online Motor
		displayPutStr("No motors detected! ",44,12,0,RED,BACKGROUND);
	}

}

