/*
 * Este archivo contiene las funciones de UART
 *
 * Josue Alejandro Gutierrez Alvarez
 */


#include "UART.h"

uint8_t UART0Buffer[128];
uint8_t UART0BufferStart;
uint8_t UART0BufferEnd;

uint32_t baudcheck;

void initUART0(uint32_t baudrate){

	baudcheck = baudrate;

		SIM->SOPT2 |= SIM_SOPT2_LPUARTSRC(1);
		SIM->SCGC2 |= SIM_SCGC2_LPUART0(1);	// UART0 Clock enable

		SIM->SCGC5 |= SIM_SCGC5_PORTE(1);	// PORTE Clock enable

		PORTD->PCR[8] |= PORT_PCR_MUX(5);	// LPUART0_TX ALT5
		PORTD->PCR[9] |= PORT_PCR_MUX(5);	// LPUART0_RX ALT5

		LPUART0->CTRL &= ~(1<<LPUART_CTRL_TE_SHIFT);
		LPUART0->CTRL &= ~(1<<LPUART_CTRL_RE_SHIFT);

		LPUART0->BAUD =LPUART_BAUD_OSR(0x0F);
		LPUART0->BAUD = LPUART_BAUD_SBR((((96000000/8/baudrate) + 1)/2));

		LPUART0->CTRL &= ~LPUART_CTRL_M_MASK;
		LPUART0->CTRL &= ~LPUART_CTRL_PE_MASK;
		LPUART0->BAUD &= ~LPUART_BAUD_SBNS_MASK;  // One stop bit

		LPUART0->CTRL |= LPUART_CTRL_TE_MASK;
		LPUART0->CTRL |= LPUART_CTRL_RE_MASK;
		LPUART0->CTRL |= LPUART_CTRL_RIE_MASK;	// Enable interrupt for NVIC Handler

		NVIC_SetPriority(LPUART0_IRQn,0);
		NVIC_EnableIRQ(LPUART0_IRQn);

		UART0BufferStart = 0;
		UART0BufferEnd = 0;
}

void UART0Write(uint8_t data){
	while (!(LPUART0->STAT & (1<<LPUART_STAT_TDRE_SHIFT)));
	LPUART0->DATA = data;
	while (!(LPUART0->STAT & (1<<LPUART_STAT_TC_SHIFT)));
}

void UART0WriteString(char *dataString){
	while(*dataString != 0){
		UART0Write(*dataString++);
	}
}

uint8_t UART0Read(void){
	if(UART0Available()){
		return UART0Buffer[UART0BufferStart++];
	}else{
		return -1;
	}
}

uint8_t UART0Peek(void){
	return UART0Buffer[UART0BufferStart];
}

uint8_t UART0Available(void){
	return (UART0BufferEnd + 1) - UART0BufferStart;
}

void UART0Flush(void){
	UART0BufferStart = 0;
	UART0BufferEnd = 0;
}

void LPUART0_IRQHandler(void){
	if(LPUART0->STAT & (1<<LPUART_STAT_RDRF_SHIFT)){
		UART0Buffer[UART0BufferEnd] = LPUART0->DATA;
		UART0BufferEnd++;
	}
}
